unit FileHistory;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Menus;

type TItemClickEvent = procedure (Sender: TObject) of object;

type
  TFileHistory = class(TComponent)
  private
    FCaption        :string;
    FHisCount       :smallint;
    FBreakBefore    :Boolean;
    FBreakAfter     :Boolean;
    FMenu           :TMainMenu;
    FAfterMenuItem  :TMenuItem;
    FHistory        :TStrings;
    fmnu_indx       :integer;
    FOnItemClick    :TItemClickEvent;
    FSelected       :string;
    procedure SetHistory(Value:TStrings);
    function menuitemindex(amenuitem :Tmenuitem):integer;
    procedure sethiscount(value :smallint);
    procedure ItemClick(Sender: TObject);
    function getitem(value :string):string;
  protected
    { Protected declarations }
  public
    constructor create(AOwner :TComponent); override;
    procedure SetMenu;
    procedure AddHistory(value :string);
    procedure DeleteHistory(value :string);
    procedure SaveToFile(FileName :string);
    procedure LoadFromFile(FileName :string);
  published
    property History:TStrings read FHistory write SetHistory;
    property Menu :TMainMenu read FMenu write FMenu;
    property AfterMenuItem :TMenuItem read FAfterMenuItem write FAfterMenuItem;
    property BreakAfter :Boolean read Fbreakafter write fbreakafter;
    property BreakBefore :Boolean read Fbreakbefore write fbreakbefore;
    property Caption :string read fcaption write fcaption;
    property HisCount :smallint read fhiscount write sethiscount;
    property OnItemClick : TItemClickEvent read FOnItemClick write FOnItemClick;
    property Selected :string read fselected;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Smart', [TFileHistory]);
end;

{ TFileHistory }

constructor TFileHistory.create(AOwner: TComponent);
var  mi         :TMenuItem;
begin
  inherited create(AOwner);
  fhistory:=tstringlist.create;
  hiscount:=0;
  fmnu_indx:=-1;
end;

procedure TFileHistory.SetHistory(Value :TStrings);
begin
  FHistory.assign(Value);
end;

function TFileHistory.menuitemindex(amenuitem :Tmenuitem):integer;
var i   :integer;
begin
  result:=-1;
  for i:=0 to menu.Items[0].count-1 do
  begin
    if amenuitem.name = menu.Items[0].items[i].name then
    begin
      result:=i+1;
      exit;
    end;
  end;
end;


procedure TFileHistory.SetMenu;
var
  mi         :TMenuItem;
  ami_ndx,i  :integer;
begin

  if (fmnu_indx <> -1) then
  begin
    menu.Items[0].items[fmnu_indx].free;
  end;

  if History.count=0 then
  begin
    fmnu_indx:=-1;
    exit;
  end;

  ami_ndx:=menuitemindex(AfterMenuItem);
  if breakbefore then
  begin
    mi:=tmenuitem.create(self);
    mi.Caption:='-';
    menu.Items[0].insert(ami_ndx,mi);
    inc(ami_ndx);
  end;

  mi:=tmenuitem.create(self);
  mi.Caption:=Caption;
  menu.Items[0].insert(ami_ndx,mi);
  fmnu_indx:=ami_ndx;

  for i:=0 to History.count-1 do
  begin
    mi:=tmenuitem.create(self);
    mi.Caption:=inttostr(i) + '  ' + history[i];
    menu.Items[0].items[ami_ndx].add(mi);
    mi.OnClick:=ItemClick;
  end;

  if breakafter then
  begin
    inc(ami_ndx);
    mi:=tmenuitem.create(self);
    mi.Caption:='-';
    menu.Items[0].insert(ami_ndx,mi);
  end;

end;

procedure TFileHistory.AddHistory(value: string);
var i     :smallint;
begin

  for i:=0 to history.count-1 do
  if uppercase(history[i])= uppercase(value) then
    exit;

  history.Add(value);

  if history.Count > hiscount then
    history.Delete(0);

  setmenu;
end;

procedure TFileHistory.sethiscount(value: smallint);
begin
  fhiscount:=value;
end;

procedure TFileHistory.DeleteHistory(value: string);
var i     :integer;
begin
  for i:=0 to history.count-1 do
  begin
    if uppercase(history[i]) = uppercase(value) then
    begin
      history.Delete(i);
      setmenu;
      exit;
    end;
  end;
end;

procedure TFileHistory.ItemClick(Sender: TObject);
begin
  fselected:=getitem(tmenuitem(sender).caption);

  if assigned(FOnItemClick) then
    FOnItemClick(self);
end;


function TFileHistory.getitem(value: string): string;
var i     :integer;
    l     :string;
    p     :pchar;
    s     :pchar;
begin
  for i:=1 to length(value) do
  begin
    if value[i]=' ' then
    begin
      result:=trim(copy(value,i,length(value)-i+1));
      exit;
    end;
  end;
  result:='';
end;

procedure TFileHistory.SaveToFile(FileName: string);
var afile   :textfile;
    i       :integer;
begin
  assignfile(afile,filename);
  rewrite(afile);
  for i:=0 to history.count-1 do
    writeln(afile,history[i]);
  closefile(afile);
end;

procedure TFileHistory.LoadFromFile(FileName: string);
var afile   :textfile;
    l       :string;
begin
  assignfile(afile,filename);
  {$I-} reset(afile) {$I+};
  if ioresult=0 then
  begin
    history.clear;
    while not eof(afile) do
    begin
      readln(afile,l);
      history.add(l);
    end;
    closefile(afile);
    setmenu;
  end;  
end;

end.
